@@ -14,12 +14,21 @@
 /*
 *   MACROS
 */
-#ifndef PROGRAM_VERSION
-# define PROGRAM_VERSION "5.0.1"
+#if defined (HAVE_CONFIG_H)
+# define PROGRAM_VERSION PACKAGE_VERSION
+#else
+# define PROGRAM_VERSION "0.0.0"
 #endif
-#define PROGRAM_NAME    "Exuberant Ctags"
-#define PROGRAM_URL     "http://ctags.sourceforge.net"
-#define AUTHOR_NAME     "Darren Hiebert"
-#define AUTHOR_EMAIL    "darren@hiebert.com"
+#define PROGRAM_NAME      "Universal Ctags"
+#define PROGRAM_URL       "https://ctags.io/"
+#define PROGRAM_COPYRIGHT "Copyright (C) 2015"
+#define AUTHOR_NAME       "Universal Ctags Team"
+
+/*
+ * Constant
+ */
+extern const char* ctags_repoinfo;
+#define CTAGS_FIELD_PREFIX "UCTAGS"
+
 
 #endif	/* CTAGS_MAIN_CTAGS_H */
