@@ -77,7 +77,10 @@
 # include <gtk/gtk-a11y.h>
 #endif
 
-#if defined(__WIN32__) || defined(_MSC_VER)
+#if defined(_WIN32)
+// On Win32 use windows.h to access CLIPFORMAT
+#undef NOMINMAX
+#define NOMINMAX
 #include <windows.h>
 #endif
 
@@ -92,6 +95,7 @@
 #include "SciLexer.h"
 #endif
 #include "StringCopy.h"
+#include "CharacterCategory.h"
 #ifdef SCI_LEXER
 #include "LexerModule.h"
 #endif
@@ -147,7 +151,7 @@ ScintillaGTKAccessible *ScintillaGTKAccessible::FromAccessible(GtkAccessible *ac
 	// FIXME: do we need the check below?  GTK checks that in all methods, so maybe
 	GtkWidget *widget = gtk_accessible_get_widget(accessible);
 	if (! widget) {
-		return 0;
+		return nullptr;
 	}
 
 	return SCINTILLA_OBJECT_ACCESSIBLE_GET_PRIVATE(accessible)->pscin;
@@ -163,16 +167,16 @@ ScintillaGTKAccessible::ScintillaGTKAccessible(GtkAccessible *accessible_, GtkWi
 
 ScintillaGTKAccessible::~ScintillaGTKAccessible() {
 	if (gtk_accessible_get_widget(accessible)) {
-		g_signal_handlers_disconnect_matched(sci->sci, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, this);
+		g_signal_handlers_disconnect_matched(sci->sci, G_SIGNAL_MATCH_DATA, 0, 0, nullptr, nullptr, this);
 	}
 }
 
 gchar *ScintillaGTKAccessible::GetTextRangeUTF8(Sci::Position startByte, Sci::Position endByte) {
-	g_return_val_if_fail(startByte >= 0, NULL);
+	g_return_val_if_fail(startByte >= 0, nullptr);
 	// FIXME: should we swap start/end if necessary?
-	g_return_val_if_fail(endByte >= startByte, NULL);
+	g_return_val_if_fail(endByte >= startByte, nullptr);
 
-	gchar *utf8Text = NULL;
+	gchar *utf8Text = nullptr;
 	const char *charSetBuffer;
 
 	// like TargetAsUTF8, but avoids a double conversion
@@ -207,7 +211,7 @@ gchar *ScintillaGTKAccessible::GetText(int startChar, int endChar) {
 
 gchar *ScintillaGTKAccessible::GetTextAfterOffset(int charOffset,
 		AtkTextBoundary boundaryType, int *startChar, int *endChar) {
-	g_return_val_if_fail(charOffset >= 0, NULL);
+	g_return_val_if_fail(charOffset >= 0, nullptr);
 
 	Sci::Position startByte, endByte;
 	Sci::Position byteOffset = ByteOffsetFromCharacterOffset(charOffset);
@@ -249,7 +253,7 @@ gchar *ScintillaGTKAccessible::GetTextAfterOffset(int charOffset,
 
 		default:
 			*startChar = *endChar = -1;
-			return NULL;
+			return nullptr;
 	}
 
 	CharacterRangeFromByteRange(startByte, endByte, startChar, endChar);
@@ -258,7 +262,7 @@ gchar *ScintillaGTKAccessible::GetTextAfterOffset(int charOffset,
 
 gchar *ScintillaGTKAccessible::GetTextBeforeOffset(int charOffset,
 		AtkTextBoundary boundaryType, int *startChar, int *endChar) {
-	g_return_val_if_fail(charOffset >= 0, NULL);
+	g_return_val_if_fail(charOffset >= 0, nullptr);
 
 	Sci::Position startByte, endByte;
 	Sci::Position byteOffset = ByteOffsetFromCharacterOffset(charOffset);
@@ -311,7 +315,7 @@ gchar *ScintillaGTKAccessible::GetTextBeforeOffset(int charOffset,
 
 		default:
 			*startChar = *endChar = -1;
-			return NULL;
+			return nullptr;
 	}
 
 	CharacterRangeFromByteRange(startByte, endByte, startChar, endChar);
@@ -320,7 +324,7 @@ gchar *ScintillaGTKAccessible::GetTextBeforeOffset(int charOffset,
 
 gchar *ScintillaGTKAccessible::GetTextAtOffset(int charOffset,
 		AtkTextBoundary boundaryType, int *startChar, int *endChar) {
-	g_return_val_if_fail(charOffset >= 0, NULL);
+	g_return_val_if_fail(charOffset >= 0, nullptr);
 
 	Sci::Position startByte, endByte;
 	Sci::Position byteOffset = ByteOffsetFromCharacterOffset(charOffset);
@@ -373,7 +377,7 @@ gchar *ScintillaGTKAccessible::GetTextAtOffset(int charOffset,
 
 		default:
 			*startChar = *endChar = -1;
-			return NULL;
+			return nullptr;
 	}
 
 	CharacterRangeFromByteRange(startByte, endByte, startChar, endChar);
@@ -383,7 +387,7 @@ gchar *ScintillaGTKAccessible::GetTextAtOffset(int charOffset,
 #if ATK_CHECK_VERSION(2, 10, 0)
 gchar *ScintillaGTKAccessible::GetStringAtOffset(int charOffset,
 		AtkTextGranularity granularity, int *startChar, int *endChar) {
-	g_return_val_if_fail(charOffset >= 0, NULL);
+	g_return_val_if_fail(charOffset >= 0, nullptr);
 
 	Sci::Position startByte, endByte;
 	Sci::Position byteOffset = ByteOffsetFromCharacterOffset(charOffset);
@@ -405,7 +409,7 @@ gchar *ScintillaGTKAccessible::GetStringAtOffset(int charOffset,
 		}
 		default:
 			*startChar = *endChar = -1;
-			return NULL;
+			return nullptr;
 	}
 
 	CharacterRangeFromByteRange(startByte, endByte, startChar, endChar);
@@ -527,10 +531,10 @@ static AtkAttributeSet *AddTextColorAttribute(AtkAttributeSet *attributes, AtkTe
 }
 
 AtkAttributeSet *ScintillaGTKAccessible::GetAttributesForStyle(unsigned int styleNum) {
-	AtkAttributeSet *attr_set = NULL;
+	AtkAttributeSet *attr_set = nullptr;
 
 	if (styleNum >= sci->vs.styles.size())
-		return NULL;
+		return nullptr;
 	Style &style = sci->vs.styles[styleNum];
 
 	attr_set = AddTextAttribute(attr_set, ATK_TEXT_ATTR_FAMILY_NAME, g_strdup(style.fontName));
@@ -547,7 +551,7 @@ AtkAttributeSet *ScintillaGTKAccessible::GetAttributesForStyle(unsigned int styl
 }
 
 AtkAttributeSet *ScintillaGTKAccessible::GetRunAttributes(int charOffset, int *startChar, int *endChar) {
-	g_return_val_if_fail(charOffset >= -1, NULL);
+	g_return_val_if_fail(charOffset >= -1, nullptr);
 
 	Sci::Position byteOffset;
 	if (charOffset == -1) {
@@ -557,7 +561,7 @@ AtkAttributeSet *ScintillaGTKAccessible::GetRunAttributes(int charOffset, int *s
 	}
 	int length = sci->pdoc->Length();
 
-	g_return_val_if_fail(byteOffset <= length, NULL);
+	g_return_val_if_fail(byteOffset <= length, nullptr);
 
 	const char style = StyleAt(byteOffset, true);
 	// compute the range for this style
@@ -583,7 +587,7 @@ gint ScintillaGTKAccessible::GetNSelections() {
 
 gchar *ScintillaGTKAccessible::GetSelection(gint selection_num, int *startChar, int *endChar) {
 	if (selection_num < 0 || (unsigned int) selection_num >= sci->sel.Count())
-		return NULL;
+		return nullptr;
 
 	Sci::Position startByte = sci->sel.Range(selection_num).Start().Position();
 	Sci::Position endByte = sci->sel.Range(selection_num).End().Position();
@@ -737,7 +741,7 @@ void ScintillaGTKAccessible::PasteText(int charPosition) {
 		Sci::Position bytePosition;
 
 		void Destroyed() override {
-			scia = 0;
+			scia = nullptr;
 		}
 
 		Helper(ScintillaGTKAccessible *scia_, Sci::Position bytePos_) :
@@ -763,7 +767,7 @@ void ScintillaGTKAccessible::PasteText(int charPosition) {
 		static void TextReceivedCallback(GtkClipboard *clipboard, const gchar *text, gpointer data) {
 			Helper *helper = static_cast<Helper*>(data);
 			try {
-				if (helper->scia != 0) {
+				if (helper->scia != nullptr) {
 					helper->TextReceived(clipboard, text);
 				}
 			} catch (...) {}
@@ -917,20 +921,20 @@ void ScintillaGTKAccessible::Notify(GtkWidget *, gint, SCNotification *nt) {
 
 // AtkText
 gchar *ScintillaGTKAccessible::AtkTextIface::GetText(AtkText *text, int start_offset, int end_offset) {
-	WRAPPER_METHOD_BODY(text, GetText(start_offset, end_offset), NULL);
+	WRAPPER_METHOD_BODY(text, GetText(start_offset, end_offset), nullptr);
 }
 gchar *ScintillaGTKAccessible::AtkTextIface::GetTextAfterOffset(AtkText *text, int offset, AtkTextBoundary boundary_type, int *start_offset, int *end_offset) {
-	WRAPPER_METHOD_BODY(text, GetTextAfterOffset(offset, boundary_type, start_offset, end_offset), NULL)
+	WRAPPER_METHOD_BODY(text, GetTextAfterOffset(offset, boundary_type, start_offset, end_offset), nullptr)
 }
 gchar *ScintillaGTKAccessible::AtkTextIface::GetTextBeforeOffset(AtkText *text, int offset, AtkTextBoundary boundary_type, int *start_offset, int *end_offset) {
-	WRAPPER_METHOD_BODY(text, GetTextBeforeOffset(offset, boundary_type, start_offset, end_offset), NULL)
+	WRAPPER_METHOD_BODY(text, GetTextBeforeOffset(offset, boundary_type, start_offset, end_offset), nullptr)
 }
 gchar *ScintillaGTKAccessible::AtkTextIface::GetTextAtOffset(AtkText *text, gint offset, AtkTextBoundary boundary_type, gint *start_offset, gint *end_offset) {
-	WRAPPER_METHOD_BODY(text, GetTextAtOffset(offset, boundary_type, start_offset, end_offset), NULL)
+	WRAPPER_METHOD_BODY(text, GetTextAtOffset(offset, boundary_type, start_offset, end_offset), nullptr)
 }
 #if ATK_CHECK_VERSION(2, 10, 0)
 gchar *ScintillaGTKAccessible::AtkTextIface::GetStringAtOffset(AtkText *text, gint offset, AtkTextGranularity granularity, gint *start_offset, gint *end_offset) {
-	WRAPPER_METHOD_BODY(text, GetStringAtOffset(offset, granularity, start_offset, end_offset), NULL)
+	WRAPPER_METHOD_BODY(text, GetStringAtOffset(offset, granularity, start_offset, end_offset), nullptr)
 }
 #endif
 gunichar ScintillaGTKAccessible::AtkTextIface::GetCharacterAtOffset(AtkText *text, gint offset) {
@@ -952,16 +956,16 @@ void ScintillaGTKAccessible::AtkTextIface::GetCharacterExtents(AtkText *text, gi
 	WRAPPER_METHOD_BODY(text, GetCharacterExtents(offset, x, y, width, height, coords), )
 }
 AtkAttributeSet *ScintillaGTKAccessible::AtkTextIface::GetRunAttributes(AtkText *text, gint offset, gint *start_offset, gint *end_offset) {
-	WRAPPER_METHOD_BODY(text, GetRunAttributes(offset, start_offset, end_offset), NULL)
+	WRAPPER_METHOD_BODY(text, GetRunAttributes(offset, start_offset, end_offset), nullptr)
 }
 AtkAttributeSet *ScintillaGTKAccessible::AtkTextIface::GetDefaultAttributes(AtkText *text) {
-	WRAPPER_METHOD_BODY(text, GetDefaultAttributes(), NULL)
+	WRAPPER_METHOD_BODY(text, GetDefaultAttributes(), nullptr)
 }
 gint ScintillaGTKAccessible::AtkTextIface::GetNSelections(AtkText *text) {
 	WRAPPER_METHOD_BODY(text, GetNSelections(), 0)
 }
 gchar *ScintillaGTKAccessible::AtkTextIface::GetSelection(AtkText *text, gint selection_num, gint *start_pos, gint *end_pos) {
-	WRAPPER_METHOD_BODY(text, GetSelection(selection_num, start_pos, end_pos), NULL)
+	WRAPPER_METHOD_BODY(text, GetSelection(selection_num, start_pos, end_pos), nullptr)
 }
 gboolean ScintillaGTKAccessible::AtkTextIface::AddSelection(AtkText *text, gint start, gint end) {
 	WRAPPER_METHOD_BODY(text, AddSelection(start, end), FALSE)
@@ -1000,7 +1004,7 @@ static GType scintilla_object_accessible_factory_get_type(void);
 
 static void scintilla_object_accessible_init(ScintillaObjectAccessible *accessible);
 static void scintilla_object_accessible_class_init(ScintillaObjectAccessibleClass *klass);
-static gpointer scintilla_object_accessible_parent_class = NULL;
+static gpointer scintilla_object_accessible_parent_class = nullptr;
 
 
 // @p parent_type is only required on GTK 3.2 to 3.6, and only on the first call
@@ -1010,27 +1014,27 @@ static GType scintilla_object_accessible_get_type(GType parent_type G_GNUC_UNUSE
 	if (g_once_init_enter(&type_id_result)) {
 		GTypeInfo tinfo = {
 			0,															/* class size */
-			(GBaseInitFunc) NULL,										/* base init */
-			(GBaseFinalizeFunc) NULL,									/* base finalize */
+			(GBaseInitFunc) nullptr,										/* base init */
+			(GBaseFinalizeFunc) nullptr,									/* base finalize */
 			(GClassInitFunc) scintilla_object_accessible_class_init,	/* class init */
-			(GClassFinalizeFunc) NULL,									/* class finalize */
-			NULL,														/* class data */
+			(GClassFinalizeFunc) nullptr,									/* class finalize */
+			nullptr,														/* class data */
 			0,															/* instance size */
 			0,															/* nb preallocs */
 			(GInstanceInitFunc) scintilla_object_accessible_init,		/* instance init */
-			NULL														/* value table */
+			nullptr														/* value table */
 		};
 
 		const GInterfaceInfo atk_text_info = {
 			(GInterfaceInitFunc) ScintillaGTKAccessible::AtkTextIface::init,
-			(GInterfaceFinalizeFunc) NULL,
-			NULL
+			(GInterfaceFinalizeFunc) nullptr,
+			nullptr
 		};
 
 		const GInterfaceInfo atk_editable_text_info = {
 			(GInterfaceInitFunc) ScintillaGTKAccessible::AtkEditableTextIface::init,
-			(GInterfaceFinalizeFunc) NULL,
-			NULL
+			(GInterfaceFinalizeFunc) nullptr,
+			nullptr
 		};
 
 #if HAVE_GTK_A11Y_H
@@ -1071,13 +1075,13 @@ static GType scintilla_object_accessible_get_type(GType parent_type G_GNUC_UNUSE
 }
 
 static AtkObject *scintilla_object_accessible_new(GType parent_type, GObject *obj) {
-	g_return_val_if_fail(SCINTILLA_IS_OBJECT(obj), NULL);
+	g_return_val_if_fail(SCINTILLA_IS_OBJECT(obj), nullptr);
 
 	AtkObject *accessible = (AtkObject *) g_object_new(scintilla_object_accessible_get_type(parent_type),
 #if HAVE_WIDGET_SET_UNSET
 		"widget", obj,
 #endif
-		NULL);
+		nullptr);
 	atk_object_initialize(accessible, obj);
 
 	return accessible;
@@ -1089,7 +1093,7 @@ static AtkObject *scintilla_object_accessible_new(GType parent_type, GObject *ob
 // @p cache pointer to store the AtkObject between repeated calls.  Might or might not be filled.
 // @p widget_parent_class pointer to the widget's parent class (to chain up method calls).
 AtkObject *ScintillaGTKAccessible::WidgetGetAccessibleImpl(GtkWidget *widget, AtkObject **cache, gpointer widget_parent_class G_GNUC_UNUSED) {
-	if (*cache != NULL) {
+	if (*cache != nullptr) {
 		return *cache;
 	}
 
@@ -1139,7 +1143,7 @@ static AtkStateSet *scintilla_object_accessible_ref_state_set(AtkObject *accessi
 	AtkStateSet *state_set = ATK_OBJECT_CLASS(scintilla_object_accessible_parent_class)->ref_state_set(accessible);
 
 	GtkWidget *widget = gtk_accessible_get_widget(GTK_ACCESSIBLE(accessible));
-	if (widget == NULL) {
+	if (widget == nullptr) {
 		atk_state_set_add_state(state_set, ATK_STATE_DEFUNCT);
 	} else {
 		if (! scintilla_send_message(SCINTILLA_OBJECT(widget), SCI_GETREADONLY, 0, 0))
@@ -1159,11 +1163,11 @@ static AtkStateSet *scintilla_object_accessible_ref_state_set(AtkObject *accessi
 
 static void scintilla_object_accessible_widget_set(GtkAccessible *accessible) {
 	GtkWidget *widget = gtk_accessible_get_widget(accessible);
-	if (widget == NULL)
+	if (widget == nullptr)
 		return;
 
 	ScintillaObjectAccessiblePrivate *priv = SCINTILLA_OBJECT_ACCESSIBLE_GET_PRIVATE(accessible);
-	if (priv->pscin != 0)
+	if (priv->pscin)
 		delete priv->pscin;
 	priv->pscin = new ScintillaGTKAccessible(accessible, widget);
 }
@@ -1171,7 +1175,7 @@ static void scintilla_object_accessible_widget_set(GtkAccessible *accessible) {
 #if HAVE_WIDGET_SET_UNSET
 static void scintilla_object_accessible_widget_unset(GtkAccessible *accessible) {
 	GtkWidget *widget = gtk_accessible_get_widget(accessible);
-	if (widget == NULL)
+	if (widget == nullptr)
 		return;
 
 	ScintillaObjectAccessiblePrivate *priv = SCINTILLA_OBJECT_ACCESSIBLE_GET_PRIVATE(accessible);
@@ -1195,7 +1199,7 @@ static void scintilla_object_accessible_finalize(GObject *object) {
 
 	if (priv->pscin) {
 		delete priv->pscin;
-		priv->pscin = 0;
+		priv->pscin = nullptr;
 	}
 
 	G_OBJECT_CLASS(scintilla_object_accessible_parent_class)->finalize(object);
@@ -1224,7 +1228,7 @@ static void scintilla_object_accessible_class_init(ScintillaObjectAccessibleClas
 static void scintilla_object_accessible_init(ScintillaObjectAccessible *accessible) {
 	ScintillaObjectAccessiblePrivate *priv = SCINTILLA_OBJECT_ACCESSIBLE_GET_PRIVATE(accessible);
 
-	priv->pscin = 0;
+	priv->pscin = nullptr;
 }
 
 #if HAVE_GTK_FACTORY
