@@ -8,9 +8,7 @@
 #ifndef EDITMODEL_H
 #define EDITMODEL_H
 
-#ifdef SCI_NAMESPACE
 namespace Scintilla {
-#endif
 
 /**
 */
@@ -24,10 +22,6 @@ public:
 };
 
 class EditModel {
-	// Private so EditModel objects can not be copied
-	explicit EditModel(const EditModel &);
-	EditModel &operator=(const EditModel &);
-
 public:
 	bool inOverstrike;
 	int xOffset;		///< Horizontal scrolled amount in pixels
@@ -36,7 +30,7 @@ public:
 	SpecialRepresentations reprs;
 	Caret caret;
 	SelectionPosition posDrag;
-	Position braces[2];
+	Sci::Position braces[2];
 	int bracesMatchStyle;
 	int highlightGuideColumn;
 	Selection sel;
@@ -45,10 +39,12 @@ public:
 	enum IMEInteraction { imeWindowed, imeInline } imeInteraction;
 
 	int foldFlags;
-	ContractionState cs;
+	int foldDisplayTextStyle;
+	UniqueString defaultFoldDisplayText;
+	std::unique_ptr<IContractionState> pcs;
 	// Hotspot support
 	Range hotspot;
-	int hoverIndicatorPos;
+	Sci::Position hoverIndicatorPos;
 
 	// Wrapping support
 	int wrapWidth;
@@ -56,15 +52,21 @@ public:
 	Document *pdoc;
 
 	EditModel();
+	// Deleted so EditModel objects can not be copied.
+	EditModel(const EditModel &) = delete;
+	EditModel(EditModel &&) = delete;
+	EditModel &operator=(const EditModel &) = delete;
+	EditModel &operator=(EditModel &&) = delete;
 	virtual ~EditModel();
-	virtual int TopLineOfMain() const = 0;
+	virtual Sci::Line TopLineOfMain() const = 0;
 	virtual Point GetVisibleOriginInMain() const = 0;
-	virtual int LinesOnScreen() const = 0;
-	virtual Range GetHotSpotRange() const = 0;
+	virtual Sci::Line LinesOnScreen() const = 0;
+	virtual Range GetHotSpotRange() const noexcept = 0;
+	void SetDefaultFoldDisplayText(const char *text);
+	const char *GetDefaultFoldDisplayText() const noexcept;
+	const char *GetFoldDisplayText(Sci::Line lineDoc) const;
 };
 
-#ifdef SCI_NAMESPACE
 }
-#endif
 
 #endif
