@@ -463,49 +463,25 @@ void filetypes_init(void)
 }
 
 
-/* Find a filetype that predicate returns TRUE for, otherwise return NULL. */
-GeanyFiletype *filetypes_find(GCompareFunc predicate, gpointer user_data)
+static guint match_basename(const GeanyFiletype *ft, const gchar *base_filename)
 {
-	guint i;
-
-	for (i = 0; i < filetypes_array->len; i++)
-	{
-		GeanyFiletype *ft = filetypes[i];
-
-		if (predicate(ft, user_data))
-			return ft;
-	}
-	return NULL;
-}
-
-
-static gboolean match_basename(gconstpointer pft, gconstpointer user_data)
-{
-	const GeanyFiletype *ft = pft;
-	const gchar *base_filename = user_data;
-	gint j;
-	gboolean ret = FALSE;
-
 	if (G_UNLIKELY(ft->id == GEANY_FILETYPES_NONE))
-		return FALSE;
+		return 0;
 
-	for (j = 0; ft->pattern[j] != NULL; j++)
+	for (guint j = 0; ft->pattern[j] != NULL; j++)
 	{
-		GPatternSpec *pattern = g_pattern_spec_new(ft->pattern[j]);
-
-		if (g_pattern_match_string(pattern, base_filename))
+		gchar *pat = ft->pattern[j];
+		
+		if (g_pattern_match_simple(pat, base_filename))
 		{
-			ret = TRUE;
-			g_pattern_spec_free(pattern);
-			break;
+			return strlen(pat);
 		}
-		g_pattern_spec_free(pattern);
 	}
-	return ret;
+	return 0;
 }
 
 
-static GeanyFiletype *check_builtin_filenames(const gchar *utf8_filename)
+static GeanyFiletype *detect_filetype_conf_file(const gchar *utf8_filename)
 {
 	gchar *lfn = NULL;
 	gchar *path;
@@ -539,8 +515,9 @@ GeanyFiletype *filetypes_detect_from_extension(const gchar *utf8_filename)
 {
 	gchar *base_filename;
 	GeanyFiletype *ft;
+	guint plen = 0;
 
-	ft = check_builtin_filenames(utf8_filename);
+	ft = detect_filetype_conf_file(utf8_filename);
 	if (ft)
 		return ft;
 
@@ -551,7 +528,21 @@ GeanyFiletype *filetypes_detect_from_extension(const gchar *utf8_filename)
 	SETPTR(base_filename, g_utf8_strdown(base_filename, -1));
 #endif
 
-	ft = filetypes_find(match_basename, base_filename);
+	for (guint i = 0; i < filetypes_array->len; i++)
+	{
+		guint mlen = match_basename(filetypes[i], base_filename);
+		
+		if (mlen > plen)
+		{	// longest pattern match wins
+			plen = mlen;
+			ft = filetypes[i];
+		}
+		else if (mlen == plen && ft && !ft->priv->user_extensions &&
+			filetypes[i]->priv->user_extensions)
+		{	// user config overrides system if pattern len same
+			ft = filetypes[i];
+		}
+	}
 	if (ft == NULL)
 		ft = filetypes[GEANY_FILETYPES_NONE];
 
@@ -1397,6 +1388,7 @@ static void read_extensions(GKeyFile *sysconfig, GKeyFile *userconfig)
 		gchar **list = g_key_file_get_string_list(
 			(userset) ? userconfig : sysconfig, "Extensions", filetypes[i]->name, &len, NULL);
 
+		filetypes[i]->priv->user_extensions = userset;
 		g_strfreev(filetypes[i]->pattern);
 		/* Note: we allow 'Foo=' to remove all patterns */
 		if (!list)
