@@ -1,8 +1,7 @@
 /*
  *      socket.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2006 The Geany contributors
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -23,8 +22,11 @@
 #ifndef GEANY_SOCKET_H
 #define GEANY_SOCKET_H 1
 
+#include <glib.h>
 
-struct socket_info_struct
+G_BEGIN_DECLS
+
+struct SocketInfo
 {
 	gboolean	 ignore_socket;
 	gchar		*file_name;
@@ -33,7 +35,7 @@ struct socket_info_struct
 	guint 		 lock_socket_tag;
 };
 
-extern struct socket_info_struct socket_info;
+extern struct SocketInfo socket_info;
 
 gint socket_init(gint argc, gchar **argv);
 
@@ -41,5 +43,6 @@ gboolean socket_lock_input_cb(GIOChannel *source, GIOCondition condition, gpoint
 
 gint socket_finalize(void);
 
+G_END_DECLS
 
-#endif
+#endif /* GEANY_SOCKET_H */
